--=============================================================
--           Words Pad for Renoid v.1.01  
--         Design:Satobox  Coding:Uprime22 
-- (http://reg.s63.xrea.com/BBS/viewtopic.php?f=2&t=524&start=0 )
--================================================================
--[[
========================
main.lua
========================
--]]


local vb = renoise.ViewBuilder()

--[[ データ　]]------------------------
  -- 以下の、表、行、段の３次元で表されるデータに合わせて、ボタンを配置します。
  -- （上下ボタンも追加しました）
  
  -- 50音等表示データtable。
  -- 例：tbl_ons[1][2][3]は「く」。
  local tbl_ons =
  {
    {
      {"あ","い","う","え","お"},
      {"か","き","く","け","こ"},
      {"さ","し","す","せ","そ"},
      {"た","ち","つ","て","と"},
      {"な","に","ぬ","ね","の"},
      {"は","ひ","ふ","へ","ほ"},
      {"ま","み","む","め","も"},
      {"や","","ゆ","いぇ","よ"},
      {"ら","り","る","れ","ろ"},
      {"わ","ん","OFF"}
    },
    {  
      {"が","ぎ","ぐ","げ","ご"},
      {"ざ","じ","ず","ぜ","ぞ"},
      {"だ","でぃ","どぅ","で","ど"},
      {"ば","び","ぶ","べ","ぼ"},
      {"ぱ","ぴ","ぷ","ぺ","ぽ"}
    },
    {
      {"▲","▼"}
    }
  }
 
  -- 対応する音階等のデータ。
  -- 'C-0' - 'G-9', 'OFF' or '---'。
  -- （'up','down'を追加。）
  -- 上のtbl_onsに合わせる必要がある。
  local tbl_notes =
  {
    {
      {"C-4","C#4","D-4","D#4","E-4"},
      {"F-4","F#4","G-4","G#4","A-4"},
      {"C-5","C#5","D-5","D#5","E-5"},
      {"F-5","F#5","G-5","G#5","A-5"},
      {"C-6","C#6","D-6","D#6","E-6"},
      {"F-6","F#6","G-6","G#6","A-6"},
      {"C-7","C#7","D-7","D#7","E-7"},
      {"C-8","---","C#8","D-8","D#8",},
      {"F-7","F#7","G-7","G#7","A-7"},
      {"E-8","F-8","OFF"}
    },
    {
      {"F-1","F#1","G-1","G#1","A-1"},
      {"C-2","C#2","D-2","D#2","E-2"},
      {"F-2","F#2","G-2","G#2","A-2"},
      {"C-3","C#3","D-3","D#3","E-3"},
      {"F-3","F#3","G-3","G#3","A-3"}
    },
    {
      {"up","down"}
    }
  }
---------------------------------------

  
  -- 以下、tableの長さの取得。#でなくtable.maxnを使う事。
  
  -- 表の数。入れ子になったtbl_onsの中の最上層のtableの数。
 function hmax()
   return table.maxn(tbl_ons)
 end

  -- 行の数。h番目の表の中の行数。
 function gmax(h)
   return table.maxn(tbl_ons[h])
 end

  -- 段の数。h番目の表中のg番目の行の中のボタン数。
 function dmax(h,g)
   return table.maxn(tbl_ons[h][g])
 end




-- カーソルを動かす関数定義を呼び出す。
-- Thank you,KMaki,joule!
-- http://forum.renoise.com/index.php?/topic/32240-snippet-edit-step/
require ("move_cursor")


-- ここにボタン押下時動作する関数を定義
-- 変数はstringで。'C-0' - 'G-9', 'OFF' or '---'。
-- とれる変数に'up','down'を追加。
-- 'up','down'は、edit stepに呼応して動くようにしてみました。
function note_it (note)
  local rs=renoise.song()
  local nc = rs.selected_note_column
  if rs.transport.edit_mode == true
  then
    if note =="down" then move_cursor()
    elseif note =="up" then move_cursor(- rs.transport.edit_step)
    elseif (nc ~= nil) then
      nc.note_string = note     
      -- instrument_indexは-1しないとinstrument_valueにならないのに注意。
      -- 255は空白。
      if note =="---" then
        nc.instrument_value =255
      else
        nc.instrument_value =
        rs.selected_instrument_index -1
      end
       move_cursor()
    else   
    end
  else return nil
  end
 end



  -- あるtableに対応する構造を持つ、新たな空の入れ子tableを用意する関数
  -- 多次元配列の用意のため。
  function set_ini(tbl)
    local new ={}
    for i,val in pairs(tbl) do
      if type(val) ~= "table" then
        new[i] = nil
      else
        new[i] = set_ini(val)
      end
    end
    return new
  end
  


-- note column 監視。notifierの練習になる。
require('indexer4jagged_etc')

  -- note2index['あ']->{1,1,1} みたいに
  -- Indexをくれるテーブルを作成。
  local note2index =
  indexer4jagged (tbl_notes)

  function note2on (note)
    local on
    local idx = note2index[note]
    on =return_val(tbl_ons,idx)
    return on
  end

  -- この変数WatchはGUI立ち上げ関数の中に入れないと、エラーになる。
  -- コメントアウトしときます。
--[[
  local watch =
  vb:text {
    align = "center",
    id = "note_catched",
    text = " ",
    font = "Big"
  }
 --]]
 -- id 'note_catched'であるようなViewのText属性に、
 -- 読み取った情報を変換したものをリンクさせる関数
 local note_str
  function watch_notes ()
   local rs=renoise.song()
   local nc =rs.selected_note_column
   if (nc ~= nil) then
     note_str = nc.note_string
   end
 end
 function catch_notes ()
   if (note_str ~= nil) then
     vb.views.note_catched.text=tostring(note2on(note_str))
   end
 end

-- 上のwatch_notes()をアイドリングさせる関数
function run()
  if not (renoise.tool().app_idle_observable:has_notifier(watch_notes)) then
    renoise.tool().app_idle_observable:add_notifier(watch_notes)
    renoise.tool().app_idle_observable:add_notifier(catch_notes)
  end
end
-- stopさせる。
function stop()
  if (renoise.tool().app_idle_observable:has_notifier(watch_notes)) then
    renoise.tool().app_idle_observable:remove_notifier(watch_notes)
    renoise.tool().app_idle_observable:remove_notifier(catch_notes)
    vb.views.note_catched = nil

  end
end



-- GUI立ち上げ用関数
function on_buttons ()

  stop()   

  -- tbl_onsに合わせて、多次元の配列用意。
  -- s0は全体、s1は表の集まり、s2は行の集まり、s3はボタンの集まり、にしようとしてる。
  -- 例：s1[1]は1番目の表。s2[1][2]は,か行。s3[1][2][3]は「く」のボタン。
  
  -- その前に初期化。
  -- 面倒なんでs0以外全部同じ次元で。
  local s0
  local s1 = set_ini(tbl_ons)
  local s2 = set_ini(tbl_ons)
  local s3 = set_ini(tbl_ons)
  



  -- s3にボタンを入れていく。
  for h = 1,hmax() do
    for g = 1,gmax(h) do
      for d = 1,dmax(h,g) do
        s3[h][g][d] =
        vb:button ({
          width = 35,
          height = 35,
          text = tbl_ons[h][g][d],
          notifier =
          function()
            note_it(tbl_notes[h][g][d])
          end
        })
      end
    end
  end


  -- s2にs3の要素を横に並べた塊（Rack）を入れていき,
  -- 行（Rack）の集まりとする。
  for h = 1,hmax() do
    for g = 1,gmax(h) do
        s2[h][g] =
        vb:row(s3[h][g])
    end
  end


  -- s1にs2の要素を縦に並べた塊（Rack）を入れていき
  -- 表（Rack)の集まりとする。
  for h = 1,hmax() do
    s1[h] =
    vb:column(s2[h])
  end

-- 読み取ったNoteを入れるView。
  local watch =
  vb:text {
    align = "center",
    id = "note_catched",
    text = " ",
    font = "Big"
  }

  -- s0にs1の要素を縦や横に並べた塊（Rack)や、上のwatchを入れて、全体とする。
  -- （例：s1[3]は上下のボタンの一行の表（Rack)です。）
  -- s1の要素（表）の間をあけたい時は
  -- spacingを挟んだり。
  s0 = vb:row{s1[1], spacing =12, vb:column{s1[2],spacing =20,s1[3],spacing =20,watch}}

  -- 表示。
  renoise.app():show_custom_dialog("Words Pad for Renoid" ,s0)
  
  -- Note監視用notifierを起動させる。
  run()
end


-- menu entries登録。            
renoise.tool():add_menu_entry {
   name = "Pattern Editor:Words Pad for Renoid...",
   invoke = function() on_buttons() end
}
-- test
--[[
renoise.tool():add_menu_entry {
   name = "Pattern Editor:test Words Pad...",
   invoke = function() stop() end
}
--]]
